/*
 *  XNap
 *
 *  A pure java file sharing client.
 *
 *  See AUTHORS for copyright information.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package xnap.plugin.gift.net.event;

import xnap.plugin.gift.net.lexer.Command;


/**
 * DebugEvent
 *
 * @author <a href="mailto:tvanlessen@taval.de">Tammo van Lessen</a>
 * @version CVS $Id: DebugEvent.java,v 1.1 2003/01/15 16:17:53 squig Exp $
 */
public class DebugEvent extends Event {
    //~ Static fields/initializers ---------------------------------------------

    public static final int RECEIVE = 0;
    public static final int SEND = 1;
    public static final String[] TYPES = { "Received", "Sent" };

    //~ Instance fields --------------------------------------------------------

    private Command cmd;
    private int type;

    //~ Constructors -----------------------------------------------------------

    /**
     * Constructor for DebugEvent.
     *
     * @param type
     * @param cmd
     */
    public DebugEvent(int type, Command cmd) {
        super();
        this.type = type;
        this.cmd = cmd;
    }

    //~ Methods ----------------------------------------------------------------

    /**
     * Returns the command.
     *
     * @return Command
     */
    public Command getCmd() {
        return cmd;
    }

    /**
     * Returns the type.
     *
     * @return int
     */
    public int getType() {
        return type;
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return TYPES[type] + "_" + cmd.getCommand() + "@DebugEvent";
    }
}
