/*
 *  XNap
 *
 *  A pure java file sharing client.
 *
 *  See AUTHORS for copyright information.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package xnap.plugin.gift.net.event;

/**
 * ErrorEvent
 *
 * @author <a href="mailto:tvanlessen@taval.de">Tammo van Lessen</a>
 * @version CVS $Id: ErrorEvent.java,v 1.1 2003/01/15 16:17:54 squig Exp $
 */
public class ErrorEvent extends Event {
    //~ Instance fields --------------------------------------------------------

    private Exception e;
    private String msg;

    //~ Constructors -----------------------------------------------------------

    /**
     * Constructor for ErrorEvent.
     *
     * @param msg Message
     */
    public ErrorEvent(String msg) {
        super();
        this.msg = msg;
        this.e = null;
    }

    /**
     * Creates a new ErrorEvent object.
     *
     * @param msg Message
     * @param e Exception
     */
    public ErrorEvent(String msg, Exception e) {
        super();
        this.msg = msg;
        this.e = e;
    }

    /**
     * Creates a new ErrorEvent object.
     *
     * @param e Exception
     */
    public ErrorEvent(Exception e) {
        super();
        this.msg = null;
        this.e = e;
    }

    //~ Methods ----------------------------------------------------------------

    /**
     * Returns the msg.
     *
     * @return String
     */
    public String getErrorMessage() {
        return msg;
    }

    /**
     * Returns the exception.
     *
     * @return Exception
     */
    public Exception getException() {
        return e;
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return "Network Error (" + getErrorMessage() + ")";
    }
}
